/*
 * Copyright 2013 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MouseDriver.h"
#include <cmath>
#include <limits>

MouseDriver::MouseDriver()
{
	for (int i = 0; i < 4; ++i)
	{
		buttons[i] = 0;
	}
}

int MouseDriver::drive(int, const Track&)
{	
	int flags = 0;
	Uint8 state = SDL_GetMouseState(0, 0);
	for (int i = 0; i < 4; ++i)
	{
		if (state & buttons[i])
		{
			flags |= 1 << i;
		}
	}
	return flags;
}

bool MouseDriver::setControl(const SDL_Event& event, int control)
{
	
	if (event.type == SDL_MOUSEBUTTONDOWN &&
		event.button.button > 0 && 
		event.button.button <= 8 &&
		event.button.button != SDL_BUTTON_WHEELUP &&
		event.button.button != SDL_BUTTON_WHEELDOWN)
	{
		buttons[control] = SDL_BUTTON(event.button.button);
		return true;
	}
	return false;
}

std::string MouseDriver::getControlName(int control) const
{	
	switch (buttons[control])
	{
	case 0:
		return "";
	case SDL_BUTTON(SDL_BUTTON_LEFT):
		return "left button";
	case SDL_BUTTON(SDL_BUTTON_MIDDLE):
		return "middle button";
	case SDL_BUTTON(SDL_BUTTON_RIGHT):
		return "right button";
	case SDL_BUTTON(SDL_BUTTON_WHEELUP):
		return "wheel up";
	case SDL_BUTTON(SDL_BUTTON_WHEELDOWN):
		return "wheel down";
	case SDL_BUTTON(6):
		return "extra button 1";
	case SDL_BUTTON(7):
		return "extra button 2";
	case SDL_BUTTON(8):
		return "extra button 3";
	default:
		return "unknown button";
	}
}

void MouseDriver::save(std::ostream& out) const
{
	out << "Mouse\n";
	for (int i = 0; i < 4; ++i)
	{
		out << ' ' << buttons[i];
	}
	out << '\n';
}

void MouseDriver::load(std::istream& in)
{
	for (int i = 0; i < 4; ++i)
	{
		in >> buttons[i];
	}
	in.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
}
